/*
 * Mach Operating System
 * Copyright (c) 1992 Carnegie Mellon University
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */

| Assembly-language entry stubs for exception routines
|
| Contributors:
|       Bryan Ford
|       Markus Wild


	.text
	.even


| general fault, currently invoked just for buserror

	.globl _trap
_trap:
	illegal
	btst   #5,sp@
	jeq     Luser_trap
	| we're in supervisor mode then
	moveml  d0-d1/a0-a1,sp@-
	jsr     _kernel_trap_c
	moveml	sp@+,d0-d1/a0-a1
	rte
Luser_trap:
	moveml  d0-d7/a0-a6,sp@-
	movel   usp,a0
	movel   a0,sp@-
	movel   sp,a0
	movel   _active_stacks,a1
	lea     a1@(8192-4),sp
	movel   a0,sp@
	| no return here ever??? 					XXX
	jmp     _user_trap_c


| FIXME: Are we supposed to get syscalls in kernel mode? |/

kernel_syscall:
	moveml  d0-d7/a0-a6,sp@-
	movel   usp,a0
	movel   a0,sp@-
	movel   sp,sp@-
	movel   d0,sp@-
	| no return here ever???					XXX
	jmp     _mach_msg_reg_c

	.globl	_syscall
_syscall:
	illegal
	btst    #5,sp@
	jne     kernel_syscall
	moveml  d0-d7/a0-a6,sp@-
	movel   usp,a0
	movel   a0,sp@-
	movel   sp,a0
	movel   _active_stacks,a1
	lea     a1@(8192-4),sp
	movel   a0,sp@
	movel   d0,sp@-
	| no return here ever???					XXX
	jmp     _syscall_c


| FIXME: Are we supposed to get syscalls in kernel mode? |/

kernel_mach_msg_reg:
	moveml  d0-d7/a0-a6,sp@-
	movel   usp,a0
	movel   a0,sp@-
	movel   sp,sp@-
	| no return here ever???					XXX
	jmp     _mach_msg_reg_c

	.globl	_mach_msg_reg
_mach_msg_reg:
	illegal
	btst    #5,sp@
	jne     kernel_mach_msg_reg
	moveml  d0-d7/a0-a6,sp@-
	movel   usp,a0
	movel   a0,sp@-
	movel   sp,a0
	movel   _active_stacks,a1
	lea     a1@(8192-4),sp
	movel   a0,sp@
	| no return here ever???					XXX
	jmp     _mach_msg_reg_c


	.globl	_interrupt
_interrupt:
	moveml	d0-d1/a0-a1,sp@-

	moveq	#0,d0
	btst	#5,sp@(4*4)
	sne	d0				| arg!=0 => kernel-int
	movel	d0,sp@-				| pass as argument

	| frame@(6) looks like this if we interrupted a user-mode thread or another interrupt:
	| 0 0 0 0 vector-offset(12 bit)
	| looks like this if we interrupted a kernel-mode thread:
	| 0 0 0 1 vector-offset(12 bit)
	| (this is a throwaway stack frame; the real one is on the master stack.)
	| spurious-interrupt has vector-offset 0x60
	movew   sp@(4*4+4+6),d0                 | index into int_table
	andiw   #0x0fff,d0
	movel   pc@(int_table-.-2-0x60:b,d0:w),a0 | fetch C handler
	jsr	a0@				| and call it
	lea	sp@(4),sp			| pop argument

	moveml	sp@+,d0-d1/a0-a1
	rte

int_table:
	.long	_spurious_int
	.long	_level_1_int_c                  | These are C functions in
	.long	_level_2_int_c                  | system-dependent code
	.long	_level_3_int_c                  | (typically in interrupt.c)
	.long	_level_4_int_c
	.long	_level_5_int_c
	.long	_level_6_int_c
	.long	_level_7_int


	| we don't currently deal with them, so shortcut them here..
_spurious_int:
	| some boards seem to generate spurious NMIs, ignore them
_level_7_int:
	rts


        | Called to continue a new thread.  Described in more detail in pcb.c
        .globl _Thread_continue
_Thread_continue:
        movel d0,sp@(4)                         | Previously running thread
        jra a2@


